import os
import sys
import json
import requests
from .cli import CLI
from datetime import datetime

class GraymetaClient():

    def __init__(self, server_url, api_key):
        self.SERVER_URL = server_url
        self.API_KEY = api_key
        self.HEADERS = { "Authorization": "Bearer " + self.API_KEY }
        self.verbose = False

    def summary_platform(self):
        return self.http_get("/api/data/summary/platform")

    def summary_data(self):
        return self.http_get("/api/data/summary/data")

    def extract_all(self, cli):
        """
        Extracts all item metadata to disk
        """

        # cache file file is the set of items already downloaded *not* to download again
        cache_filename = cli.getOrDie("-cache_file")
        if os.path.isfile(cache_filename):
            cache = json.loads(open(cache_filename, 'r').read())
        else:
            cache = []

        # a little cache so I don't have to re-extract
        output_dir = cli.getOrDie("-output_dir")
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        # search file is a results of a previous search call so we don't
        # have to call search again.  Use if present.
        if cli.containsKey("-search_file"):
            print("Not calling search, using file")
            search_filename = cli.getOrDie("-search_file")
            search_response = json.loads(open(search_filename, 'r').read())
            print("Loaded search file ok.")
        else:
            print("Calling search")
            search_response = self.search()
            print("Called search, now extracting results")


        total_results = len(search_response["results"])
        print("Retrieved " + str(total_results) + " results.")
        for r in search_response["results"]:
            start_time = datetime.today()
            result = r["result"]
            stow_url = result["stow_url"]
            filename = stow_url.split("/")[-1]
            execution_id = stow_url.split("/")[-3] + "/" + stow_url.split("/")[-2]
            gm_item_id = result["_id"]
            if gm_item_id in cache:
                print("Already processed " + gm_item_id + ", not extracting again.")
                continue

            if not os.path.exists(output_dir + "/" + execution_id):
                os.makedirs(output_dir + "/" + execution_id)

            if "name" in result:
                # then it has been harvested
                gm_item = self.get_gm_item(gm_item_id)
                gm_item_v2 = self.get_gm_item_v2(gm_item_id)

                f1 = open(output_dir + "/" + execution_id + "/" + filename + "_v1.json", "w")
                f1.write(json.dumps(gm_item, indent=4))
                f1.close()

                f2 = open(output_dir + "/" + execution_id + "/" + filename + "_v2.json", "w")
                f2.write(json.dumps(gm_item_v2, indent=4))
                f2.close()

                f3 = open(output_dir + "/" + execution_id + "/" + filename + "_index.json", "w")
                f3.write(json.dumps(result, indent=4))
                f3.close()

                cache.append(gm_item_id)

                open(cache_filename, 'w').write(json.dumps(cache, indent=4))
                ttl = (datetime.today() - start_time).seconds
                print("Processed " + gm_item_id + " in " + str(ttl) + "s.")
            else:
                print(gm_item_id + " not harvested yet.")




        print("Called search ok, now fetching content.")
        print("Extract Complete")

    def extract(self, cli):
        """
        Extracts all items whose stow_url matches a search term
        """

        search_term = cli.getOrDie("-q")
        print("Extracting all items with '" + search_term + "' in their stow_url.")

        # search file is a results of a previous search call so we don't
        # have to call search again.  Use if present.
        if cli.containsKey("-search_file"):
            print("Not calling search, using file")
            search_filename = cli.getOrDie("-search_file")
            search_response = json.loads(open(search_filename, 'r').read())
            print("Loaded search file ok.")
        else:
            print("Calling search")
            search_response = self.search()
            print("Called search, now extracting results")

        # a little cache so I don't have to re-extract
        output_dir = cli.getOrDie("-output_dir")
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        for r in search_response["results"]:
            start_time = datetime.today()
            result = r["result"]
            stow_url = result["stow_url"]

            should_download = stow_url.find(search_term) > -1

            filename = stow_url.split("/")[-1]
            execution_id = stow_url.split("/")[-3] + "/" + stow_url.split("/")[-2]
            gm_item_id = result["_id"]

            if not should_download:
                continue

            if not os.path.exists(output_dir + "/" + execution_id):
                os.makedirs(output_dir + "/" + execution_id)

            if "name" in result:
                # then it has been harvested
                gm_item = self.get_gm_item(gm_item_id)
                gm_item_v2 = self.get_gm_item_v2(gm_item_id)

                f1 = open(output_dir + "/" + execution_id + "/" + filename + "_v1.json", "w")
                f1.write(json.dumps(gm_item, indent=4))
                f1.close()

                f2 = open(output_dir + "/" + execution_id + "/" + filename + "_v2.json", "w")
                f2.write(json.dumps(gm_item_v2, indent=4))
                f2.close()

                f3 = open(output_dir + "/" + execution_id + "/" + filename + "_index.json", "w")
                f3.write(json.dumps(result, indent=4))
                f3.close()

                ttl = (datetime.today() - start_time).seconds
                print("Processed " + gm_item_id + " in " + str(ttl) + "s.")
            else:
                print(gm_item_id + " not harvested yet.")

        print("Extract Complete")


    def features(self):
        return self.http_get("/api/data/features")

    def add_comment(self, gm_item_id, comment):
        url = "/api/data/comments"
        data = { "target_type": "item", "target_id": gm_item_id, "body": comment }
        return self.http_post(url, data)

    def list_comments(self, gm_item_id):
        url = "/api/data/comments?target_type=item&page=0&target_id=" + gm_item_id
        return self.http_get(url)

    def delete_comment(self, gm_item_id, comment_id):
        url = "/api/data/comments/" + comment_id
        return self.http_delete(url)

    def harvest_item(self, location_id, container_id, stow_url, gm_item_id, force, extractors, override_extractors, new_extractors):
        """
        POST /api/control/harvest
        {
        	"location_id": "{location_id}",
        	"item_id": "{item_id}",
        	"force": {force}
        }
        """
        url = "/api/control/harvest"
        # data = { "location_id": location_id, "container_id": container_id, "stow_url": stow_url, "gm_item_id": gm_item_id, "force": force, "override_extractors": False}
        # data = { "location_id": location_id, "item_id": gm_item_id, "force": force, "override_extractors": False, }
        data = { "location_id": location_id, "container_id": container_id, "item_stow_url": stow_url, "force": force, "extractors": extractors, "override_extractors": override_extractors, "new_extractors": new_extractors }
        print("harvest_item: url=" + url + ", data=" + str(data))
        return self.http_post(url, data)

    def harvest_container(self, location_id, container_id, force=False):
        """
        POST /api/control/harvest
        {
        	"location_id": "{location_id}",
        	"container_id": "{container_id}",
        	"force": {force}
        }
        """
        url = "/api/control/harvest"
        data = { "location_id": location_id, "container_id": container_id, "force": force}
        return self.http_post(url, data)

    def create_gm_item_id_from_s3_key(self, s3_key):
        """
        The asset has not yet been walked, this assigns a gm_item_id to it without harvesting
        POST /api/control/item-id
        {"location_id":"xxxx", "container_id":"xxxxx", "item_id":"filename.txt"}

        returns

        {'gm_item_id': 'xxxx', 'stow_url': 's3://https://xxxxx/content.mp4'}

        """

        print("gmapi.create_gm_item_id_from_s3_key(" + s3_key + ")")
        key = s3_key.replace("s3://", "")
        splits = key.split("/")
        bucket = splits[0]
        filename = "/".join(splits[1:])

        locations = self.list_locations()
        location_id = locations["locations"][0]["id"]
        containers = self.list_containers(location_id)

        container = None
        for candidate_container in containers["containers"]:
            if candidate_container["id"] == bucket:
                container_id = candidate_container["id"]
                break

        url = "/api/control/item-id"
        post_data = { "location_id": location_id, "container_id": container_id, "item_id": filename }
        print(post_data)
        return self.http_post(url, post_data)

    def get_gm_item_from_s3_key(self, s3_key):
        gm_item_id, location_id = self.get_gm_item_id_from_s3_key(s3_key)
        if gm_item_id != None:
            item_data = self.get_gm_item_id(str(location_id), str(container["id"]), str(filename))
            item = self.get_gm_item(gm_item_id)
            return item
        else:
            return None


    def get_gm_item_id_from_s3_key(self, s3_key):
        key = s3_key.replace("s3://", "")
        splits = key.split("/")
        bucket = splits[0]
        filename = "/".join(splits[1:])

        locations = self.list_locations()
        location_id = locations["locations"][0]["id"]
        containers = self.list_enabled_containers()

        container = None
        for candidate_container in containers:
            if candidate_container["id"] == bucket:
                container = candidate_container
                break

        if container is None:
            print("No container found matching '" + bucket + "'")
            sys.exit(1)
        else:
            item_data = self.get_gm_item_id(str(location_id), str(container["id"]), str(filename))
            if item_data != None:
                gm_item_id = item_data["gm_item_id"]
                return gm_item_id, location_id
            else:
                return None, None


    def get_gm_item_id(self, location_id, container_id, item_id):
        url = self.SERVER_URL + "/api/control/item-id"
        data = { "location_id": location_id, "container_id": container_id, "item_id": item_id }
        data_str = json.dumps(data)
        headers = self.HEADERS
        self._setupDebug()
        r = requests.post(url, data=data_str, headers=headers)
        if r.status_code >= 200 and r.status_code <= 299:
            return r.json()
        else:
            return None

    def get_gm_item(self, gm_item_id):
        return self.http_get("/api/data/items/" + gm_item_id)

    def get_gm_item_v2(self, gm_item_id):
        return self.http_get("/files/" + gm_item_id + "/metadata2.json")

    def list_items(self, container_id):
        return self.http_get("/api/data/items")

    def delete_gm_item(self, gm_item_id):
        url = self.SERVER_URL + "/api/data/items/" + gm_item_id
        headers = self.HEADERS
        self._setupDebug()
        r = requests.delete(url, headers=headers)
        return r.json()

    def upload_captions(self, gm_item_id, stl_filename):
        url = self.SERVER_URL + "/api/data/items/" + gm_item_id + "/captions"
        headers = self.HEADERS
        files = { "caption_file": open(stl_filename, 'rb') }
        self._setupDebug()
        r = requests.post(url, files=files, headers=headers)
        return r.json()

    def get_captions(self, gm_item_id):
        url = "/api/data/items/" + gm_item_id + "?only=captions.captions"
        return self.http_get(url)

    def delete_captions(self, gm_item_id, captions_id):
        url = self.SERVER_URL + "/api/data/items/" + gm_item_id + "/captions?caption_id=" + captions_id
        headers = self.HEADERS
        self._setupDebug()
        r = requests.delete(url, headers=headers)
        return r.json()
  
    def list_location(self, location_id):
        return self.http_get("/api/data/locations/" + location_id)

    def list_locations(self):
        return self.http_get("/api/data/locations")

    def list_containers(self, location_id):
        return self.http_get("/api/data/locations/" + location_id + "/containers")

    def list_enabled_containers(self):
        """
        GET /api/data/containers/enabled
        """
        return self.http_get("/api/data/containers/enabled")

    def health(self):
        return self.http_get("/api/data/healthz")

    def isIdle(self):
        """ 
        indicates if GM has any work to do or is working on anything
        """
        stats = self.stats()
        running_jobs = stats.get("jobs").get("running") or 0
        pending_jobs = stats.get("jobs").get("pending") or 0
        return (running_jobs + pending_jobs) == 0

    def stats(self):
        """
{
    "queue_depths": {
        "index": 0,
        "stage09": 0,
        "stage08": 0,
        "stage05": 0,
        "stage04": 0,
        "stage07": 0,
        "stage06": 0,
        "stage01": 0,
        "stage00": 0,
        "stage03": 0,
        "stage02": 0,
        "activity": 0,
        "walk": 0
    },
    "jobs": {
        "running": 0,
        "pending": 0
    }
}
        """
        return self.http_get("/api/control/system/stats")

    def activity(self):
        return self.http_get("/api/data/activity")

    def user(self):
        return self.http_get("/api/data/user")

    def platform(self):
        return self.http_get("/api/data/summary/platform")

    def scroll(self):
        return self.http_post("/api/data/scroll", {})

    def search(self, limit=50000):
        data = { "limit": limit }
        return self.http_post("/api/data/search", data)

    def search_quick(self, limit=50000):
        data = { "limit": limit, "only": [ "gm_item_id" ] }
        return self.http_post("/api/data/search", data)

    def search_extracted(self, limit=50000):
        filters = { "exists": [ { "field": "extracted", "value": True } ] }
        data = { "limit": limit, "filters": filters }
        return self.http_post("/api/data/search", data)

    def search_not_extracted(self, limit=50000):
        filters = { "not_exists": [ { "field": "extracted", "value": True } ] }
        data = { "limit": limit, "filters": filters }
        return self.http_post("/api/data/search", data)

    def search_last_modified(self, date_from, date_to, limit=1000):
        data = { "limit": limit, "last_modified": { "from": date_from, "to": date_to } }
        return self.http_post("/api/data/search", data)

    def search_last_harvested(self, date_from, date_to, limit=1000):
        data = { "limit": limit, "last_harvested": { "from": date_from, "to": date_to } }
        return self.http_post("/api/data/search", data)

    def compilations(self):
        return self.http_get("/api/data/summary/compilations")

    def keyword_list_groups(self):
        return self.http_get("/api/data/keywords")

    def keyword_get_group(self, group_id):
        return self.http_get("/api/data/keyword-groups/" + group_id)

    def keyword_create_group(self, name, color):
        data = {"name": name, "color": color }
        return self.http_post("/api/data/keyword-groups", data)

    def keyword_delete_group(self, group_id):
        url = self.SERVER_URL + "/api/data/keyword-groups/" + group_id
        headers = self.HEADERS
        self._setupDebug()
        r = requests.delete(url, headers=headers)
        return r.json()

    def keyword_add_to_group(self, group_id, word):
        url = "/api/data/keywords/" + group_id
        data = {"word": word}
        return self.http_post(url, data)

    def keyword_remove_from_group(self, group_id, word):
        url = self.SERVER_URL + "/api/data/keywords/" + group_id + "?word=" +word
        headers = self.HEADERS
        self._setupDebug()
        r = requests.delete(url, headers=headers)
        return r.json()

    def http_get(self, partial_url):
        url = self.SERVER_URL + partial_url
        headers = self.HEADERS
        self._setupDebug()            
        r = requests.get(url, headers=headers)
        if r.status_code >= 200 and r.status_code < 300:
            return r.json()
        else:
            return None

    def http_post(self, partial_url, data):
        url = self.SERVER_URL + partial_url
        headers = self.HEADERS
        data_str = json.dumps(data)
        self._setupDebug()            
        r = requests.post(url, data=data_str, headers=headers)
        return r.json()

    def http_delete(self, partial_url, data=None):
        url = self.SERVER_URL + partial_url
        headers = self.HEADERS
        if data:
            data_str = json.dumps(data)
        else:
            data_str = ""

        self._setupDebug()            
        r = requests.delete(url, data=data_str, headers=headers)
        return r.json()

    def _setupDebug(self):
        cli = CLI(sys.argv)
        if cli.containsKey("-v") or cli.containsKey("-verbose"):
            pass
        else:
            return

        import logging
        try:
            import http.client as http_client
        except ImportError:
            # Python 2
            import httplib as http_client
        http_client.HTTPConnection.debuglevel = 1
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)
        requests_log = logging.getLogger("requests.packages.urllib3")
        requests_log.setLevel(logging.DEBUG)
        requests_log.propagate = True
        





import sys
import os
import json

# py2/3 imports fix
from .gmapi import GraymetaClient
from .cli import CLI
from .constants import *

COMMAND="gm"

def usageAndDie():
    print("gm is a tool for querying a graymeta.com installation over https")
    print("https://github.com/simonski/gm-api-python")
    print("")
    print("Usage: ")
    print("")
    print("    gm command [arguments]")
    print("")
    server_url = os.environ.get("GRAYMETA_SERVER_URL") or None
    server_key = os.environ.get("GRAYMETA_API_KEY") or None
    ljust_value = 60

    if server_url:
        print("    GRAYMETA_SERVER_URL".ljust(ljust_value) + ": " + server_url)
    else:
        print("    GRAYMETA_SERVER_URL".ljust(ljust_value) + ": unset - please `export GRAYMETA_SERVER_URL=https://your-graymeta-server`")

    if server_key:
        print("    GRAYMETA_API_KEY".ljust(ljust_value) + ": xxxxxxxx")
    else:
        print("    GRAYMETA_API_KEY".ljust(ljust_value) + ": unset - please `export GRAYMETA_API_KEY=xxxxxxx`")

    print("")
    print("The commands are:")
    print("")
    print("    list_locations".ljust(ljust_value) + "- displays all locations")
    print("    list_location {location_id}".ljust(ljust_value) + "- gets information on a specific location")
    print("")
    print("    list_containers {location_id}".ljust(ljust_value) + "- displays enabled containers")
    print("    list_all_containers {location_id}".ljust(ljust_value) + "- displays all containers")
    print("")
    print("    search -json".ljust(ljust_value) + "- displays all items (-json prints json)")
    print("           -last_modified_from|-last_modified_to")
    print("           -last_harvested_from|-last_harvested_to")
    print("")
    print("    get_gm_item_id {location_id} {container_id} {item_id}".ljust(ljust_value) + "- gets the gm_item_id for ")
    print("    get_gm_item {gm_item_id}".ljust(ljust_value) + "- gets metadata for an item using the gm_item_id")
    print("    get_gm_item_v2 {gm_item_id}".ljust(ljust_value) + "- gets metadata v2 for an item using the gm_item_id")
    print("")
    print("    create_gm_item_id_from_s3_key {s3_key}".ljust(ljust_value) + "- create gm_item_id from an s3_key")
    print("    get_gm_item_id_from_s3_key {s3_key}".ljust(ljust_value) + "- gets gm_item_id from an s3_key")
    print("    get_gm_item_from_s3_key {s3_key}".ljust(ljust_value) + "- gets metadata for an item using the s3_key")
    print("")
    print("    delete_gm_item {gm_item_id}".ljust(ljust_value) + "- deletes the metadata from graymeta")
    print("")
    print("    get_captions {gm_item_id}".ljust(ljust_value) + "- returns the captions in json")
    print("    upload_captions {gm_item_id} {stl_filename} ".ljust(ljust_value) + "- uploads and associates an STL file with content")
    print("    delete_captions {gm_item_id} {captions_id}".ljust(ljust_value) + "- deletes the captions from the item")
    print("    upload_captions_content {gm_item_id} {stl_filename} ".ljust(ljust_value) + "- uploads and associates an STL file with content")
    print("")
    print("    harvest_item_from_s3_key {s3_key} {extractors,,}".ljust(ljust_value) + "- forces a harvest for an item via its S3 key")
    print("    harvest_container {location_id} {container_id}".ljust(ljust_value) + "- forces a harvest for an entire container.")
    print("    harvest_item {gm_item_id} {extractors,,}".ljust(ljust_value) + "- forces a harvest for an item via its gm_item_id")
    print("")
    print("    comment".ljust(ljust_value) + "- uses the Graymeta Comments API")
    print("")
    print("    keyword".ljust(ljust_value) + "- uses the Graymeta Keywords API")
    print("")
    print("    extract_all".ljust(ljust_value) + "- extracts all metadata")
    print("    extract (-q term)".ljust(ljust_value) + "- extracts all metadata where 'term' is present in the stow_url")
    print("")
    print("    stats".ljust(ljust_value) + "- print current /api/control/system/stats data.")
    print("    health".ljust(ljust_value) + "- print current /api/data/healthz data.")
    print("    activity".ljust(ljust_value) + "- print current /api/data/activity data.")
    print("    version".ljust(ljust_value) + "- print current gmapi version number.")
    print("    summary_platform".ljust(ljust_value) + "- print summary information about the platform.")
    print("    summary_data".ljust(ljust_value) + "- print summary information about the data.")
    print("    get {URL}".ljust(ljust_value) + "- returns response from a GET.")
    print("")

    sys.exit(0)

def version():
    print(COMMAND + " client " + VERSION)

def main():

    if len(sys.argv) == 1:
        usageAndDie()

    command = sys.argv[1]
    if command == "version":
        version()
        sys.exit(1)

    cli = CLI(sys.argv)
    server_url = os.environ.get("GRAYMETA_SERVER_URL") or None
    server_key = os.environ.get("GRAYMETA_API_KEY") or None

    if server_url  is None or server_url.strip() == "":
        print("Error, GRAYMETA_SERVER_URL is required.")
        sys.exit(1)

    if server_key is None or server_key.strip() == "":
        print("Error, GRAYMETA_API_KEY is required.")
        sys.exit(1)

    gm = GraymetaClient(server_url, server_key)
    if cli.containsKey("-nossl"):
        gm.SSL_VERIFY = False

    if cli.containsKey("-v") or cli.containsKey("--verbose") or cli.containsKey("-verbose"):
        gm.verbose = True

    if command == "upload_captions":
        gm_item_id = sys.argv[2]
        stl_filename = sys.argv[3]
        nicePrint(gm.upload_captions(gm_item_id, stl_filename))

    elif command == "get_captions":
        gm_item_id = sys.argv[2]
        nicePrint(gm.get_captions(gm_item_id))

    elif command == "delete_captions":
        gm_item_id = sys.argv[2]
        captions_id = sys.argv[3]
        nicePrint(gm.delete_captions(gm_item_id, captions_id))

    elif command == "disable_live_harvesting":
        gm.disable_live_harvesting()

    elif command == "extract_all":
        gm.extract_all(cli)

    elif command == "extract":
        gm.extract(cli)

    elif command == "scroll":
        nicePrint(gm.scroll())

    elif command == "features":
        nicePrint(gm.features())

    elif command == "summary_platform":
        nicePrint(gm.summary_platform())

    elif command == "summary_data":
        nicePrint(gm.summary_data())

    elif command == "comment":
        cli = CLI(sys.argv)
        command = cli.getOrDefault("comment", "list")
        gm_item_id = cli.getOrDie("-gm_item_id")

        if command == "add":
            comment = cli.getOrDie("-m")
            nicePrint(gm.add_comment(gm_item_id, comment))
        elif command == "list":
            nicePrint(gm.list_comments(gm_item_id))
        elif command == "delete":
            comment_id = cli.getOrDie("-comment_id")
            nicePrint(gm.delete_comment(gm_item_id, comment_id))
            nicePrint(gm.list_comments(gm_item_id))
        else:
            print("invalid comment command - try 'add, list, delete'")

    elif command == "keyword":
        cli = CLI(sys.argv)
        command = cli.getOrDie("keyword")
        if command == "list":
            nicePrint(gm.keyword_list_groups())
        elif command == "get":
            group_id = cli.getOrDie("-group_id")
            nicePrint(gm.keyword_get_group(group_id))

        elif command == "create_group":
            name = cli.getOrDie("-name")
            color = "#" + cli.getOrDie("-color")
            nicePrint(gm.keyword_create_group(name, color))
        elif command == "delete_group":
            group_id = cli.getOrDie("-group_id")
            nicePrint(gm.keyword_delete_group(group_id))
        elif command == "add_to_group":
            group_id = cli.getOrDie("-group_id")
            word = cli.getOrDie("-word")
            nicePrint(gm.keyword_add_to_group(group_id, word))
        elif command == "remove_from_group":
            group_id = cli.getOrDie("-group_id")
            word = cli.getOrDie("-word")
            nicePrint(gm.keyword_remove_from_group(group_id, word))
        else:
            print("gm keyword (list | get | create_group | delete_group | add_to_group | remove_from_group)")

    elif command == "list_locations":
        nicePrint(gm.list_locations())

    elif command == "list_location":
        location_id = sys.argv[2]
        nicePrint(gm.list_location(location_id))

    elif command == "list_all_containers":
        location_id = sys.argv[2]
        nicePrint(gm.list_containers(location_id))

    elif command == "list_containers":
        nicePrint(gm.list_enabled_containers())

    elif command == "harvest_container":
        location_id = sys.argv[2]
        container_id = sys.argv[3]
        nicePrint(gm.harvest_container(location_id, container_id))

    elif command == "harvest_item_from_s3_key":
        s3_key = sys.argv[2]
        extractors = sys.argv[3].split(",")
        response = gm.create_gm_item_id_from_s3_key(s3_key)
        nicePrint(response)
        #gm_item_id, location_id = gm.get_gm_item_id_from_s3_key(s3_key)
        #nicePrint(gm.harvest_item(location_id, gm_item_id))

        """
        elif command == "harvest_item":
            location_id = sys.argv[2]
            gm_item_id = sys.argv[3]
            nicePrint(gm.harvest_item(location_id, gm_item_id))
        """

    elif command == "create_gm_item_id_from_s3_key":
        s3_key = sys.argv[2]
        nicePrint(gm.create_gm_item_id_from_s3_key(s3_key))

    elif command == "get_gm_item_id_from_s3_key":
        s3_key = sys.argv[2]
        gm_item_id = gm.get_gm_item_id_from_s3_key(s3_key)
        print(gm_item_id)

    elif command == "get_gm_item_id":
        location_id = sys.argv[2]
        container_id = sys.argv[3]
        item_id = sys.argv[4]
        nicePrint(gm.get_gm_item_id(location_id, container_id, item_id))

    elif command == "get_gm_item_from_s3_key":
        s3_key = sys.argv[2]
        nicePrint(gm.get_gm_item_from_s3_key(s3_key))

    elif command == "get_gm_item":
        gm_item_id = sys.argv[2]
        nicePrint(gm.get_gm_item(gm_item_id))

    elif command == "get_gm_item_v2":
        gm_item_id = sys.argv[2]
        nicePrint(gm.get_gm_item_v2(gm_item_id))

    elif command == "delete_gm_item":
        gm_item_id = sys.argv[2]
        nicePrint(gm.delete_gm_item(gm_item_id))

    elif command == "health":
        nicePrint(gm.health())

    elif command == "stats":
        nicePrint(gm.stats())

    elif command == "activity":
        nicePrint(gm.activity())

    elif command == "user":
        nicePrint(gm.user())

    elif command == "platform":
        nicePrint(gm.platform())

    elif command == "compilations":
        nicePrint(gm.compilations())

    elif command == "search_quick":
        results = gm.search_quick()
        nicePrint(results)

    elif command == "search_extracted":
        results = gm.search_extracted()

    elif command == "search_not_extracted":
        results = gm.search_not_extracted()

    elif command == "idle":
        results = gm.isIdle()
        print(results)

    elif command == "search":

        results = None
        if cli.containsKey("-last_modified_from") or cli.containsKey("-last_modified_to"):
            last_modified_from = cli.getOrDie("-last_modified_from")
            last_modified_to = cli.getOrDie("-last_modified_to")
            results = gm.search_last_modified(last_modified_from, last_modified_to)
        elif cli.containsKey("-last_harvested_from") or cli.containsKey("-last_harvested_to"):
            last_harvested_from = cli.getOrDie("-last_harvested_from")
            last_harvested_to = cli.getOrDie("-last_harvested_to")
            results = gm.search_last_harvested(last_harvested_from, last_harvested_to)
        else:
            results = gm.search()

        if cli.containsKey("-json"):
            nicePrint(results)
        else:
            if not "results" in results:
                print("No results found.")
                print(results)
            else:
                print("ItemID".ljust(35)+"Last Harvested".ljust(27) + "Last Modified".ljust(27) + "Name".ljust(20))
                for entry in results["results"]:
                    result = entry["result"]
                    gm_item_id = result["_id"]
                    container = result.get("stow_container_id") or "stow_container_id"
                    name = result.get("name") or None
                    last_modified = result.get("last_modified") or "no last modified."
                    last_harvested = result.get("last_harvested") or "no last harvested."

                    if name is not None:
                        full_name = container + "/" + name
                    else:
                        full_name = "<not harvested> ( " + result.get("stow_url") + " )"

                    print(gm_item_id.ljust(35) + last_harvested.ljust(27) + last_modified.ljust(27) + full_name.ljust(20))

    elif command == "get":
        """
        performs an authenticated GET to the API
        """
        partial_url = sys.argv[2]
        nicePrint(gm.http_get(partial_url))

    else:
        print("I don't know how to '" + command + "'")
        sys.exit(1)

def nicePrint(data):
    if data:
        print(json.dumps(data, indent=4))
    else:
        print("No data found.")



"""
Used by setup.py, code.py
It is semantic-enough, major/minor then graymeta api release
major.minor.graymeta-compatible-point-release
I put -dev on the very end to indicate this code is not published
I take -dev off the end, then publish, then increment the minor version, put dev
on the end and commit again
"""
VERSION='1.17.30'
